@@ -63,25 +63,14 @@ union long_double {
 #define getbes64(buf, off)	((int64_t)(((uint64_t)getbeu32(buf, (off)) << 32) | getbeu32(buf, (off)+4)))
 #define getbeu64(buf, off)	((uint64_t)(((uint64_t)getbeu32(buf, (off)) << 32) | getbeu32(buf, (off)+4)))
 
-#define putbe16(buf,off,w) do {putbyte(buf, (off) ,(w) >> 8); putbyte(buf, (off)+1, (w));} while (0)
-#define putbe32(buf,off,l) do {putbe16(buf, (off) ,(l) >> 16); putbe16(buf, (off)+2, (l));} while (0)
+#define putbe16(buf,off,w) do {putbyte(buf, (off), (w) >> 8); putbyte(buf, (off)+1, (w));} while (0)
+#define putbe32(buf,off,l) do {putbe16(buf, (off), (l) >> 16); putbe16(buf, (off)+2, (l));} while (0)
 
 #define getbef(buf, off)	(i_f.i = getbes32(buf, off), i_f.f)
 #define getbed(buf, off)	(l_d.l = getbes64(buf, off), l_d.d)
 
-
-/* Zodiac protocol description uses 1-origin indexing by little-endian word */
-#define get16z(buf, n)	( (buf[2*(n)-2])	\
-		| (buf[2*(n)-1] << 8))
-#define get32z(buf, n)	( (buf[2*(n)-2])	\
-		| (buf[2*(n)-1] << 8) \
-		| (buf[2*(n)+0] << 16) \
-		| (buf[2*(n)+1] << 24))
-#define getstringz(to, from, s, e)			\
-    (void)memcpy(to, from+2*(s)-2, 2*((e)-(s)+1))
-
 /* bitfield extraction */
-extern uint64_t ubits(char buf[], unsigned int, unsigned int);
-extern int64_t sbits(char buf[], unsigned int, unsigned int);
+extern uint64_t ubits(char buf[], unsigned int, unsigned int, bool);
+extern int64_t sbits(char buf[], unsigned int, unsigned int, bool);
 
 #endif /* _GPSD_BITS_H_ */
