@@ -160,7 +160,8 @@ gps_mask_t evermore_parse(struct gps_device_t * session, unsigned char *buf,
     datalen -= 2;
 
     /*@ -usedef @*/
-    buf2[0] = '\0';	/* prevent 'Assigned value is garbage or undefined' from scan-build */
+    /* prevent 'Assigned value is garbage or undefined' from scan-build */
+    memset(buf2, '\0', sizeof(buf2));
     tp = buf2;
     for (i = 0; i < (size_t) datalen; i++) {
 	*tp = *cp++;
@@ -332,8 +333,8 @@ gps_mask_t evermore_parse(struct gps_device_t * session, unsigned char *buf,
 	 * and status for each channel from the chip.  We cannot get
 	 * codephase or carrierphase.
 	 */
-#define SBITS(sat, s, l)	sbits((char *)buf, 10 + (sat*14) + s, l)
-#define UBITS(sat, s, l)	ubits((char *)buf, 10 + (sat*14) + s, l)
+#define SBITS(sat, s, l)	sbits((char *)buf, 10 + (sat*14) + s, l, false)
+#define UBITS(sat, s, l)	ubits((char *)buf, 10 + (sat*14) + s, l, false)
 	for (k = 0; k < visible; k++) {
 	    int prn = (int)UBITS(k, 4, 5);
 	    /* this is so we can tell which never got set */
