@@ -67,7 +67,6 @@ static int tsip_write(struct gps_device_t *session,
 static bool tsip_detect(struct gps_device_t *session)
 {
     char buf[BUFSIZ];
-    unsigned int n;
     bool ret = false;
     int myfd;
     fd_set fdset;
@@ -76,8 +75,6 @@ static bool tsip_detect(struct gps_device_t *session)
     char old_parity;
     unsigned int old_stopbits;
 
-    gpsd_report(LOG_PROG, "Probing TSIP\n");
-
     old_baudrate = session->gpsdata.dev.baudrate;
     old_parity = session->gpsdata.dev.parity;
     old_stopbits = session->gpsdata.dev.stopbits;
@@ -92,6 +89,7 @@ static bool tsip_detect(struct gps_device_t *session)
     /*@+ignoresigns@*/
     myfd = session->gpsdata.gps_fd;
     if (write(myfd, buf, 4) == 4) {
+	unsigned int n;
 	for (n = 0; n < 3; n++) {
 	    FD_ZERO(&fdset);
 	    FD_SET(myfd, &fdset);
@@ -961,7 +959,7 @@ static void tsip_event_hook(struct gps_device_t *session, event_t event)
     if (session->context->readonly)
 	return;
     /* FIX-ME: Resending this might not be needed on reactivation */
-    if (event == event_identified && event == event_reactivate) {
+    if (event == event_identified || event == event_reactivate) {
 	unsigned char buf[100];
 
 	/* I/O Options */
