@@ -19,12 +19,14 @@ static double bilinear(double x1, double y1, double x2, double y2, double x,
 {
     double delta;
 
-    if (y1 == y2 && x1 == x2)
+#define EQ(a, b) (fabs((a) - (b)) < 0.001)
+    if (EQ(y1, y2) && EQ(x1, x2))
 	return (z11);
-    if (y1 == y2 && x1 != x2)
+    if (EQ(y1, y2) && !EQ(x1, x2))
 	return (z22 * (x - x1) + z11 * (x2 - x)) / (x2 - x1);
-    if (x1 == x2 && y1 != y2)
+    if (EQ(x1, x2) && !EQ(y1, y2))
 	return (z22 * (y - y1) + z11 * (y2 - y)) / (y2 - y1);
+#undef EQ
 
     delta = (y2 - y1) * (x2 - x1);
 
@@ -33,14 +35,14 @@ static double bilinear(double x1, double y1, double x2, double y2, double x,
 }
 
 
-/* return geoid separtion (MSL - WGS84) in meters, given a lat/lot in degrees */
 double wgs84_separation(double lat, double lon)
+/* return geoid separation (MSL-WGS84) in meters, given a lat/lon in degrees */
 {
 #define GEOID_ROW	19
 #define GEOID_COL	37
     /* *INDENT-OFF* */
     /*@ +charint @*/
-    const char geoid_delta[GEOID_COL*GEOID_ROW]={
+    const int geoid_delta[GEOID_COL*GEOID_ROW]={
 	/* 90S */ -30,-30,-30,-30,-30,-30,-30,-30,-30,-30,-30,-30,-30,-30,-30,-30,-30,-30,-30,-30,-30,-30,-30,-30,-30,-30, -30,-30,-30,-30,-30,-30,-30,-30,-30,-30,-30,
 	/* 80S */ -53,-54,-55,-52,-48,-42,-38,-38,-29,-26,-26,-24,-23,-21,-19,-16,-12, -8, -4, -1,  1,  4,  4,  6,  5,  4,   2, -6,-15,-24,-33,-40,-48,-50,-53,-52,-53,
 	/* 70S */ -61,-60,-61,-55,-49,-44,-38,-31,-25,-16, -6,  1,  4,  5,  4,  2,  6, 12, 16, 16, 17, 21, 20, 26, 26, 22,  16, 10, -1,-16,-29,-36,-46,-55,-54,-59,-61,
@@ -70,20 +72,17 @@ double wgs84_separation(double lat, double lon)
     ilon = (int)floor((180. + lon) / 10);
 
     /* sanity checks to prevent segfault on bad data */
-    if ((ilat > 90) || (ilat < -90)) {
-	return 0.0;
-    }
-    if ((ilon > 180) || (ilon < -180)) {
-	return 0.0;
-    }
+    if ((GEOID_ROW <= ilat) || (0 > ilat) ||
+        (GEOID_COL <= ilon) || (0 > ilon))
+        return 0.0;
 
     ilat1 = ilat;
     ilon1 = ilon;
     ilat2 = (ilat < GEOID_ROW - 1) ? ilat + 1 : ilat;
     ilon2 = (ilon < GEOID_COL - 1) ? ilon + 1 : ilon;
 
-    return bilinear(ilon1 * 10. - 180., ilat1 * 10. - 90.,
-		    ilon2 * 10. - 180., ilat2 * 10. - 90.,
+    return bilinear(ilon1 * 10.0 - 180.0, ilat1 * 10.0 - 90.0,
+		    ilon2 * 10.0 - 180.0, ilat2 * 10.0 - 90.0,
 		    lon, lat,
 		    (double)geoid_delta[ilon1 + ilat1 * GEOID_COL],
 		    (double)geoid_delta[ilon2 + ilat1 * GEOID_COL],
