@@ -20,6 +20,7 @@ PERMISSIONS
 #include <stddef.h>
 
 #include "gpsd.h"
+#ifdef SOCKET_EXPORT_ENABLE
 #include "gps_json.h"
 
 /*
@@ -33,8 +34,6 @@ PERMISSIONS
 static int json_tpv_read(const char *buf, struct gps_data_t *gpsdata,
 			 /*@null@*/ const char **endptr)
 {
-    int status;
-    char tbuf[JSON_DATE_MAX+1];
     /*@ -fullinitblock @*/
     const struct json_attr_t json_attrs_1[] = {
 	/* *INDENT-OFF* */
@@ -43,8 +42,8 @@ static int json_tpv_read(const char *buf, struct gps_data_t *gpsdata,
 			         .len = sizeof(gpsdata->dev.path)},
 	{"tag",    t_string,  .addr.string = gpsdata->tag,
 			         .len = sizeof(gpsdata->tag)},
-	{"time",   t_string,  .addr.string = tbuf,
-			         .len = sizeof(tbuf)},
+	{"time",   t_time,    .addr.real = &gpsdata->fix.time,
+			         .dflt.real = NAN},
 	{"time",   t_real,    .addr.real = &gpsdata->fix.time,
 			         .dflt.real = NAN},
 	{"ept",    t_real,    .addr.real = &gpsdata->fix.ept,
@@ -80,56 +79,12 @@ static int json_tpv_read(const char *buf, struct gps_data_t *gpsdata,
     };
     /*@ +fullinitblock @*/
 
-    status = json_read_object(buf, json_attrs_1, endptr);
-
-    if (status == 0) {
-	gpsdata->status = STATUS_FIX;
-	gpsdata->set = STATUS_SET;
-	/*@-usedef@*/
-	if (isnan(gpsdata->fix.time)!=0) {
-	    if (tbuf[0] == '\0')
-		gpsdata->fix.time = NAN;
-	    else
-		gpsdata->fix.time = iso8601_to_unix(tbuf);
-	}
-	/*@+usedef@*/
-	if (isnan(gpsdata->fix.time) == 0)
-	    gpsdata->set |= TIME_SET;
-	if (isnan(gpsdata->fix.ept) == 0)
-	    gpsdata->set |= TIMERR_SET;
-	if (isnan(gpsdata->fix.longitude) == 0)
-	    gpsdata->set |= LATLON_SET;
-	if (isnan(gpsdata->fix.altitude) == 0)
-	    gpsdata->set |= ALTITUDE_SET;
-	if (isnan(gpsdata->fix.epx) == 0 && isnan(gpsdata->fix.epy) == 0)
-	    gpsdata->set |= HERR_SET;
-	if (isnan(gpsdata->fix.epv) == 0)
-	    gpsdata->set |= VERR_SET;
-	if (isnan(gpsdata->fix.track) == 0)
-	    gpsdata->set |= TRACK_SET;
-	if (isnan(gpsdata->fix.speed) == 0)
-	    gpsdata->set |= SPEED_SET;
-	if (isnan(gpsdata->fix.climb) == 0)
-	    gpsdata->set |= CLIMB_SET;
-	if (isnan(gpsdata->fix.epd) == 0)
-	    gpsdata->set |= TRACKERR_SET;
-	if (isnan(gpsdata->fix.eps) == 0)
-	    gpsdata->set |= SPEEDERR_SET;
-	if (isnan(gpsdata->fix.epc) == 0)
-	    gpsdata->set |= CLIMBERR_SET;
-	if (isnan(gpsdata->fix.epc) == 0)
-	    gpsdata->set |= CLIMBERR_SET;
-	if (gpsdata->fix.mode != MODE_NOT_SEEN)
-	    gpsdata->set |= MODE_SET;
-    }
-    return status;
+    return json_read_object(buf, json_attrs_1, endptr);
 }
 
 static int json_noise_read(const char *buf, struct gps_data_t *gpsdata,
                            /*@null@*/ const char **endptr)
 {
-    int status;
-    char tbuf[JSON_DATE_MAX+1];
     /*@ -fullinitblock @*/
     const struct json_attr_t json_attrs_1[] = {
 	/* *INDENT-OFF* */
@@ -138,8 +93,8 @@ static int json_noise_read(const char *buf, struct gps_data_t *gpsdata,
 			         .len = sizeof(gpsdata->dev.path)},
 	{"tag",    t_string,  .addr.string = gpsdata->tag,
 			         .len = sizeof(gpsdata->tag)},
-	{"time",   t_string,  .addr.string = tbuf,
-			         .len = sizeof(tbuf)},
+	{"time",   t_time,    .addr.real = &gpsdata->gst.utctime,
+			         .dflt.real = NAN},
 	{"time",   t_real,    .addr.real = &gpsdata->gst.utctime,
 			         .dflt.real = NAN},
 	{"rms",    t_real,    .addr.real = &gpsdata->gst.rms_deviation,
@@ -161,28 +116,13 @@ static int json_noise_read(const char *buf, struct gps_data_t *gpsdata,
     };
     /*@ +fullinitblock @*/
 
-    status = json_read_object(buf, json_attrs_1, endptr);
-    if (status != 0)
-	return status;
-
-    /*@-usedef@*/
-    if (isnan(gpsdata->fix.time)!=0) {
-	if (tbuf[0] == '\0')
-	    gpsdata->gst.utctime = NAN;
-	else
-	    gpsdata->gst.utctime = iso8601_to_unix(tbuf);
-    }
-    /*@+usedef@*/
-
-    gpsdata->set |= GST_SET;
-    return 0;
+    return json_read_object(buf, json_attrs_1, endptr);
 }
 
 static int json_sky_read(const char *buf, struct gps_data_t *gpsdata,
 			 /*@null@*/ const char **endptr)
 {
     bool usedflags[MAXCHANNELS];
-    char tbuf[JSON_DATE_MAX+1];
     /*@ -fullinitblock @*/
     const struct json_attr_t json_attrs_2_1[] = {
 	/* *INDENT-OFF* */
@@ -201,9 +141,9 @@ static int json_sky_read(const char *buf, struct gps_data_t *gpsdata,
 	                             .len = sizeof(gpsdata->dev.path)},
 	{"tag",	       t_string,  .addr.string  = gpsdata->tag,
 	                             .len = sizeof(gpsdata->tag)},
-	{"time",       t_string,  .addr.string = tbuf,
-			             .len = sizeof(tbuf)},
-	{"time",       t_real,    .addr.real = &gpsdata->fix.time,
+	{"time",       t_time,    .addr.real = &gpsdata->skyview_time,
+	      	                     .dflt.real = NAN},
+	{"time",       t_real,    .addr.real = &gpsdata->skyview_time,
 	      	                     .dflt.real = NAN},
 	{"hdop",       t_real,    .addr.real    = &gpsdata->dop.hdop,
 	                             .dflt.real = NAN},
@@ -238,14 +178,6 @@ static int json_sky_read(const char *buf, struct gps_data_t *gpsdata,
     if (status != 0)
 	return status;
 
-    /*@-usedef@*/
-    if (isnan(gpsdata->fix.time)!=0) {
-	if (tbuf[0] == '\0')
-	    gpsdata->skyview_time = NAN;
-	else
-	    gpsdata->skyview_time = iso8601_to_unix(tbuf);
-    }
-    /*@+usedef@*/
     gpsdata->satellites_used = 0;
     gpsdata->satellites_visible = 0;
     (void)memset(gpsdata->used, '\0', sizeof(gpsdata->used));
@@ -258,7 +190,6 @@ static int json_sky_read(const char *buf, struct gps_data_t *gpsdata,
 	}
     }
 
-    gpsdata->set |= SATELLITE_SET;
     return 0;
 }
 
@@ -328,8 +259,9 @@ static int json_devicelist_read(const char *buf, struct gps_data_t *gpsdata,
 	{"class",      t_check,      .dflt.check = "DEVICE"},
 	{"path",       t_string,     STRUCTOBJECT(struct devconfig_t, path),
 	                                .len = sizeof(gpsdata->devices.list[0].path)},
+	{"activated",  t_time,       STRUCTOBJECT(struct devconfig_t, activated)},
 	{"activated",  t_real,       STRUCTOBJECT(struct devconfig_t, activated)},
-	{"flags",      t_integer,	   STRUCTOBJECT(struct devconfig_t, flags)},
+	{"flags",      t_integer,    STRUCTOBJECT(struct devconfig_t, flags)},
 	{"driver",     t_string,     STRUCTOBJECT(struct devconfig_t, driver),
 	                                .len = sizeof(gpsdata->devices.list[0].driver)},
 	{"subtype",    t_string,     STRUCTOBJECT(struct devconfig_t, subtype),
@@ -368,8 +300,6 @@ static int json_devicelist_read(const char *buf, struct gps_data_t *gpsdata,
     }
 
     gpsdata->devices.time = timestamp();
-    gpsdata->set &= ~UNION_SET;
-    gpsdata->set |= DEVICELIST_SET;
     return 0;
 }
 
@@ -386,6 +316,8 @@ static int json_version_read(const char *buf, struct gps_data_t *gpsdata,
 	                            .len = sizeof(gpsdata->version.rev)},
 	{"proto_major", t_integer, .addr.integer = &gpsdata->version.proto_major},
 	{"proto_minor", t_integer, .addr.integer = &gpsdata->version.proto_minor},
+	{"remote",    t_string,  .addr.string  = gpsdata->version.remote,
+	                            .len = sizeof(gpsdata->version.remote)},
 	{NULL},
 	/* *INDENT-ON* */
     };
@@ -394,12 +326,8 @@ static int json_version_read(const char *buf, struct gps_data_t *gpsdata,
 
     memset(&gpsdata->version, '\0', sizeof(gpsdata->version));
     status = json_read_object(buf, json_attrs_version, endptr);
-    if (status != 0)
-	return status;
 
-    gpsdata->set &= ~UNION_SET;
-    gpsdata->set |= VERSION_SET;
-    return 0;
+    return status;
 }
 
 static int json_error_read(const char *buf, struct gps_data_t *gpsdata,
@@ -422,9 +350,7 @@ static int json_error_read(const char *buf, struct gps_data_t *gpsdata,
     if (status != 0)
 	return status;
 
-    gpsdata->set &= ~UNION_SET;
-    gpsdata->set |= ERROR_SET;
-    return 0;
+    return status;
 }
 
 int libgps_json_unpack(const char *buf,
@@ -438,15 +364,64 @@ int libgps_json_unpack(const char *buf,
 	return -1;
 #define STARTSWITH(str, prefix)	strncmp(str, prefix, sizeof(prefix)-1)==0
     if (STARTSWITH(classtag, "\"class\":\"TPV\"")) {
-	return json_tpv_read(buf, gpsdata, end);
-    } else if (STARTSWITH(classtag, "\"class\":\"NOISE\"")) {
-	return json_noise_read(buf, gpsdata, end);
+	status = json_tpv_read(buf, gpsdata, end);
+	gpsdata->status = STATUS_FIX;
+	gpsdata->set = STATUS_SET;
+	if (isnan(gpsdata->fix.time) == 0)
+	    gpsdata->set |= TIME_SET;
+	if (isnan(gpsdata->fix.ept) == 0)
+	    gpsdata->set |= TIMERR_SET;
+	if (isnan(gpsdata->fix.longitude) == 0)
+	    gpsdata->set |= LATLON_SET;
+	if (isnan(gpsdata->fix.altitude) == 0)
+	    gpsdata->set |= ALTITUDE_SET;
+	if (isnan(gpsdata->fix.epx) == 0 && isnan(gpsdata->fix.epy) == 0)
+	    gpsdata->set |= HERR_SET;
+	if (isnan(gpsdata->fix.epv) == 0)
+	    gpsdata->set |= VERR_SET;
+	if (isnan(gpsdata->fix.track) == 0)
+	    gpsdata->set |= TRACK_SET;
+	if (isnan(gpsdata->fix.speed) == 0)
+	    gpsdata->set |= SPEED_SET;
+	if (isnan(gpsdata->fix.climb) == 0)
+	    gpsdata->set |= CLIMB_SET;
+	if (isnan(gpsdata->fix.epd) == 0)
+	    gpsdata->set |= TRACKERR_SET;
+	if (isnan(gpsdata->fix.eps) == 0)
+	    gpsdata->set |= SPEEDERR_SET;
+	if (isnan(gpsdata->fix.epc) == 0)
+	    gpsdata->set |= CLIMBERR_SET;
+	if (isnan(gpsdata->fix.epc) == 0)
+	    gpsdata->set |= CLIMBERR_SET;
+	if (gpsdata->fix.mode != MODE_NOT_SEEN)
+	    gpsdata->set |= MODE_SET;
+	return status;
+    } else if (STARTSWITH(classtag, "\"class\":\"GST\"")) {
+	status = json_noise_read(buf, gpsdata, end);
+	if (status == 0) {
+	    gpsdata->set &= ~UNION_SET;
+	    gpsdata->set |= GST_SET;
+	}
+	return status;
     } else if (STARTSWITH(classtag, "\"class\":\"SKY\"")) {
-	return json_sky_read(buf, gpsdata, end);
+	status = json_sky_read(buf, gpsdata, end);
+	if (status == 0)
+	    gpsdata->set |= SATELLITE_SET;
+	return status;
     } else if (STARTSWITH(classtag, "\"class\":\"ATT\"")) {
-	return json_att_read(buf, gpsdata, end);
+	status = json_att_read(buf, gpsdata, end);
+	if (status == 0) {
+	    gpsdata->set &= ~UNION_SET;
+	    gpsdata->set |= ATTITUDE_SET;
+	}
+	return status;
     } else if (STARTSWITH(classtag, "\"class\":\"DEVICES\"")) {
-	return json_devicelist_read(buf, gpsdata, end);
+	status = json_devicelist_read(buf, gpsdata, end);
+	if (status == 0) {
+	    gpsdata->set &= ~UNION_SET;
+	    gpsdata->set |= DEVICELIST_SET;
+	}
+	return status;
     } else if (STARTSWITH(classtag, "\"class\":\"DEVICE\"")) {
 	status = json_device_read(buf, &gpsdata->dev, end);
 	if (status == 0)
@@ -454,11 +429,18 @@ int libgps_json_unpack(const char *buf,
 	return status;
     } else if (STARTSWITH(classtag, "\"class\":\"WATCH\"")) {
 	status = json_watch_read(buf, &gpsdata->policy, end);
-	if (status == 0)
+	if (status == 0) {
+	    gpsdata->set &= ~UNION_SET;
 	    gpsdata->set |= POLICY_SET;
+	}
 	return status;
     } else if (STARTSWITH(classtag, "\"class\":\"VERSION\"")) {
-	return json_version_read(buf, gpsdata, end);
+	status = json_version_read(buf, gpsdata, end); 
+	if (status ==  0) {
+	    gpsdata->set &= ~UNION_SET;
+	    gpsdata->set |= VERSION_SET;
+	}
+	return status;
 #ifdef RTCM104V2_ENABLE
     } else if (STARTSWITH(classtag, "\"class\":\"RTCM2\"")) {
 	status = json_rtcm2_read(buf,
@@ -482,7 +464,12 @@ int libgps_json_unpack(const char *buf,
 	return status;
 #endif /* AIVDM_ENABLE */
     } else if (STARTSWITH(classtag, "\"class\":\"ERROR\"")) {
-	return json_error_read(buf, gpsdata, end);
+	status = json_error_read(buf, gpsdata, end);
+	if (status == 0) {
+	    gpsdata->set &= ~UNION_SET;
+	    gpsdata->set |= ERROR_SET;
+	}
+	return status;
     } else
 	return -1;
 #undef STARTSWITH
@@ -490,4 +477,6 @@ int libgps_json_unpack(const char *buf,
 
 /*@+compdef@*/
 
+#endif /* SOCKET_EXPORT_ENABLE */
+
 /* libgps_json.c ends here */
