@@ -95,6 +95,9 @@ PERMISSIONS
  *      GN -- Mixed GPS and GLONASS data, according to IEIC 61162-1
  *      II -- Integrated Instrumentation (Raytheon's SeaTalk system).
  *      IN -- Integrated Navigation (Garmin uses this).
+ *      WI -- Weather instrument (Airmar PB200, Radio Ocean ROWIND, Vaisala WXT520).
+ *      HC -- Heading/compass (Airmar PB200).
+ *      TI -- Turn indicator (Airmar PB200).
  *      EC -- Electronic Chart Display & Information System (ECDIS)
  *      SD -- Depth Sounder
  *      P  -- Vendor-specific sentence
@@ -324,6 +327,12 @@ static void nextstate(struct gps_packet_t *lexer, unsigned char c)
 	    lexer->state = NMEA_VENDOR_LEAD;
 	else if (c == 'I')	/* Seatalk */
 	    lexer->state = SEATALK_LEAD_1;
+	else if (c == 'W')	/* Weather instrument */
+	    lexer->state = WEATHER_LEAD_1;
+	else if (c == 'H')	/* Heading/compass */
+	    lexer->state = HEADCOMP_LEAD_1;
+	else if (c == 'T')	/* Turn indicator */
+	    lexer->state = TURN_LEAD_1;
 	else if (c == 'A')	/* SiRF Ack */
 	    lexer->state = SIRF_ACK_LEAD_1;
 	else if (c == 'E')	/* ECDIS */
@@ -500,6 +509,24 @@ static void nextstate(struct gps_packet_t *lexer, unsigned char c)
 	else
 	    lexer->state = GROUND_STATE;
 	break;
+    case WEATHER_LEAD_1:
+	if (c == 'I')		/* Weather instrument leader accepted */
+	    lexer->state = NMEA_LEADER_END;
+	else
+	    lexer->state = GROUND_STATE;
+	break;
+    case HEADCOMP_LEAD_1:
+	if (c == 'C')		/* Heading/compass leader accepted */
+	    lexer->state = NMEA_LEADER_END;
+	else
+	    lexer->state = GROUND_STATE;
+	break;
+    case TURN_LEAD_1:
+	if (c == 'I')		/* Turn indicator leader accepted */
+	    lexer->state = NMEA_LEADER_END;
+	else
+	    lexer->state = GROUND_STATE;
+	break;
     case ECDIS_LEAD_1:
 	if (c == 'C')		/* ECDIS leader accepted */
 	    lexer->state = NMEA_LEADER_END;
@@ -1989,10 +2016,12 @@ ssize_t packet_get(int fd, struct gps_packet_t *lexer)
 	return recvd;
 
     /* Otherwise, consume from the packet input buffer */
+    /* coverity[tainted_data] */
     packet_parse(lexer);
 
     /* if input buffer is full, discard */
     if (sizeof(lexer->inbuffer) == (lexer->inbuflen)) {
+	/* coverity[tainted_data] */
 	packet_discard(lexer);
 	lexer->state = GROUND_STATE;
     }
