@@ -668,10 +668,10 @@ gps_mask_t gpsd_interpret_subframe(struct gps_device_t *session,
 
 		sv = -1;
 		/* current leap seconds */
-		subp->sub4_18.alpha0 = (int8_t)((words[2] >> 16) & 0x0000FF);
+		subp->sub4_18.alpha0 = (int8_t)((words[2] >> 8) & 0x0000FF);
 		subp->sub4_18.d_alpha0 = pow(2.0, -30) * (int)subp->sub4_18.alpha0;
-		subp->sub4_18.alpha1 = (int8_t)((words[2] >>  8) & 0x0000FF);
-		subp->sub4_18.d_alpha1 = pow(2.0, -27) * (int)subp->sub4_18.alpha2;
+		subp->sub4_18.alpha1 = (int8_t)((words[2] >> 0) & 0x0000FF);
+		subp->sub4_18.d_alpha1 = pow(2.0, -27) * (int)subp->sub4_18.alpha1;
 		subp->sub4_18.alpha2 = (int8_t)((words[3] >> 16) & 0x0000FF);
 		subp->sub4_18.d_alpha2 = pow(2.0, -24) * (int)subp->sub4_18.alpha2;
 		subp->sub4_18.alpha3 = (int8_t)((words[3] >>  8) & 0x0000FF);
@@ -680,7 +680,7 @@ gps_mask_t gpsd_interpret_subframe(struct gps_device_t *session,
 		subp->sub4_18.beta0  = (int8_t)((words[3] >>  0) & 0x0000FF);
 		subp->sub4_18.d_beta0 = pow(2.0, 11) * (int)subp->sub4_18.beta0;
 		subp->sub4_18.beta1  = (int8_t)((words[4] >> 16) & 0x0000FF);
-		subp->sub4_18.d_beta1 = pow(2.0, 14) * (int)subp->sub4_18.beta2;
+		subp->sub4_18.d_beta1 = pow(2.0, 14) * (int)subp->sub4_18.beta1;
 		subp->sub4_18.beta2  = (int8_t)((words[4] >>  8) & 0x0000FF);
 		subp->sub4_18.d_beta2 = pow(2.0, 16) * (int)subp->sub4_18.beta2;
 		subp->sub4_18.beta3  = (int8_t)((words[4] >>  0) & 0x0000FF);
@@ -691,13 +691,13 @@ gps_mask_t gpsd_interpret_subframe(struct gps_device_t *session,
 		subp->sub4_18.d_A1   = pow(2.0,-50) * subp->sub4_18.A1;
 		subp->sub4_18.A0     = (int32_t)((words[6] >>  0) & 0xFFFFFF);
 		subp->sub4_18.A0   <<= 8;
-		subp->sub4_18.A0    |= ((words[7] >> 16) & 0x00FFFF);
+		subp->sub4_18.A0    |= ((words[7] >> 16) & 0x0000FF);
 		subp->sub4_18.d_A0   = pow(2.0,-30) * subp->sub4_18.A0;
 
 		/* careful WN is 10 bits, but WNt is 8 bits! */
 		/* WNt (Week Number of LSF) */
 		subp->sub4_18.tot    = ((words[7] >> 8) & 0x0000FF);
-		subp->sub4_18.d_tot  = pow(2.0,12) * subp->sub4_18.d_tot;
+		subp->sub4_18.d_tot  = pow(2.0,12) * subp->sub4_18.tot;
 		subp->sub4_18.WNt    = ((words[7] >> 0) & 0x0000FF);
 		subp->sub4_18.leap  = (int8_t)((words[8] >> 16) & 0x0000FF);
 		subp->sub4_18.WNlsf  = ((words[8] >>  8) & 0x0000FF);
@@ -706,10 +706,7 @@ gps_mask_t gpsd_interpret_subframe(struct gps_device_t *session,
 		subp->sub4_18.DN = (words[8] & 0x0000FF);
 		/* leap second future */
 		subp->sub4_18.lsf = (int8_t)((words[9] >> 16) & 0x0000FF);
-		gpsd_report(LOG_INF,
-		    "50B: SF:4-18 leap-seconds:%d lsf:%d WNlsf:%u "
-		    "DN:%d\n",
-			    subp->sub4_18.leap, subp->sub4_18.lsf, subp->sub4_18.WNlsf, subp->sub4_18.DN);
+
 		gpsd_report(LOG_PROG,
 		    "50B: SF:4-18 a0:%.5g a1:%.5g a2:%.5g a3:%.5g "
 		    "b0:%.5g b1:%.5g b2:%.5g b3:%.5g "
@@ -723,10 +720,20 @@ gps_mask_t gpsd_interpret_subframe(struct gps_device_t *session,
 			subp->sub4_18.d_tot, subp->sub4_18.WNt,
 			subp->sub4_18.leap, subp->sub4_18.WNlsf,
 			subp->sub4_18.DN, subp->sub4_18.lsf);
-		if (subp->sub4_18.leap != subp->sub4_18.lsf) {
-		    gpsd_report(LOG_PROG,
-			"50B: SF:4-18 leap-second change coming\n");
-		}
+
+#ifdef NTPSHM_ENABLE
+		if ((subp->sub4_18.WNt == subp->sub4_18.WNlsf) &&
+		    (subp->sub4_18.DN == 1 )) {
+		   if ( subp->sub4_18.leap < subp->sub4_18.lsf )
+			session->context->leap_notify = LEAP_ADDSECOND;
+		   else if ( subp->sub4_18.leap > subp->sub4_18.lsf )
+			session->context->leap_notify = LEAP_DELSECOND;
+		   else
+			session->context->leap_notify = LEAP_NOWARNING;
+		} else
+		   session->context->leap_notify = LEAP_NOWARNING;
+#endif /* NTPSHM_ENABLE */
+
 		session->context->leap_seconds = (int)subp->sub4_18.leap;
 		session->context->valid |= LEAP_SECOND_VALID;
 		break;
@@ -748,7 +755,7 @@ gps_mask_t gpsd_interpret_subframe(struct gps_device_t *session,
 	}
 	break;
     case 5:
-	/* Pages 0, dummy almanc for dummy SV 0
+	/* Pages 0, dummy almanac for dummy SV 0
 	 * Pages 1 through 24: almanac data for SV 1 through 24
 	 * Page 25: SV health data for SV 1 through 24, the almanac
 	 * reference time, the almanac reference week number.
